#' Read gene expression data (*.txt)
#'
#' readExprTxtFile' reads a tab-delimited text file containing gene expression 
#' data
#' @param fileName a string containing the file name
#' @return a numeric matrix containing the gene expression data. The  
#' rows and columns correspond to, respectively, the genes (or probe sets) and 
#' the samples.
#' @export
readExprTxtFile <- function(fileName) {
    expr <- read.table(fileName, header=TRUE)
    colnames(expr) <- toupper(colnames(expr))
    cols <- colnames(expr)
    if (cols[1] != "NAME")
        stop(paste("Wrong gene expression data format. The fist column",
        "of the file should be \"Name\"."))
    
    names <- expr[, "NAME"]
    n <- ncol(expr)

    if (cols[2] == "DESCRIPTION")
        expr <- expr[, 3:n]
    else
        expr <- expr[, 2:n]

    expr <- as.matrix(expr)
    rownames(expr) <- names
    return(expr)
}

#' Read categorical class file (*.cls)
#'
#' 'readClsFile' reads a text file delimited by blank spaces associating each
#' sample with a phenotype class.
#' @param fileName a string containing the file name
#' @param class1 a string containing the name of the first phenotype class
#' @param class2 a string containing the name of the second phenotype class
#' @return a list or vector containing the phenotype classes associated with
#' each sample. If 'class1' or 'class2' are null, then it returns a list where
#' each element represents a line of the input file (i.e. the first element 
#' contains the number of phenotype classes, the second the class names, and
#' the last the sample labels). Otherwise, it returns a vector of -1s, 0s and
#' 1s, where 0 represents the first class, 1 the second class, and -1 the other
#' classes, if the input file contains more than two classes. 
#' @export
readClsFile <- function(fileName, class1=NULL, class2=NULL) {
    lines <- readLines(fileName)
    labels <- strsplit(lines, "[ \t]+")
    if (length(labels) != 3)
        stop(paste("Wrong categorical class file format. The file should",
            "contain three rows."))
    if (length(labels[[1]]) != 3)
        stop(paste("Wrong categorical class file format. The first row",
            "should contain 3 numbers separeted by white spaces or tabs."))
    isWholeNumber <- function(x) {return((x - as.integer(x))==0)}
    if(length(which(isWholeNumber(as.numeric(labels[[1]])))) != 3) 
        stop(paste("Wrong categorical class file format. The first row",
            "should contain 3 numbers (integers)."))
    if (labels[[1]][3] != "1")
        stop(paste("Wrong categorical class file format. The third number",
            "in the first row should be one."))
    if (labels[[2]][1] != "#")
        stop(paste("Wrong categorical class file format. The first element",
            "in the second row should be \"#\"."))
    if (length(labels[[2]]) < 3 || as.numeric(labels[[1]][2]) < 2)
        stop(paste("Wrong categorical class file format. The file should",
            "contain at least 2 classes."))
    if ((length(labels[[2]]) - 1) != as.numeric(labels[[1]][2]))
        stop(paste("Wrong categorical class file format. The file indicates",
            "that there are", as.numeric(labels[[1]][2]), "classes but",
            (length(labels[[2]]) - 1), "class names were defined."))
    if ((length(labels[[2]]) - 1) != length(unique(labels[[3]])))
        stop(paste("Wrong categorical class file format. The file indicates",
            "that there are", as.numeric(labels[[1]][2]), "classes but",
             length(unique(labels[[3]])), "class names were found."))
    if (as.numeric(labels[[1]][1]) != length(labels[[3]]))
        stop(paste("Wrong categorical class file format. The file indicates",
            "that there are", as.numeric(labels[[1]][1]), "samples but",
            length(labels[[3]]), "labeled samples were found."))
    if (is.null(class1) || is.null(class2))
        return(labels)
    l <- labelsAux(labels, class1, class2)
    return(l)
}

#' Label
#'
#' 'labelsAux' creates a vector of 0s, 1s and -1s associating each sample with
#' a phenotype class.
#' @param labels: a list describing the sample labels
#' @param class1: a string containing the name of the first phenotype class
#' @param class2: a string containing the name of the second phenotype class
#' @return a vector of -1s, 0s and 1s, where 0 represents the first class, 1 the 
#' second class, and -1 the other
#' classes, if 'labels' contains more than two classes.
#' @export
labelsAux <- function(labels, class1, class2) {
    l <- array(NA, length(labels[[3]]))
    names <- labels[[2]][2:length(labels[[2]])]
    i <- which(names == class1)
    j <- which(names == class2)
    symbols <- unique(labels[[3]])
    i1 <- which(labels[[3]] == symbols[i])
    i2 <- which(labels[[3]] == symbols[j])
    l[i1] <- 0
    l[i2] <- 1 
    l[-(union(i1, i2))] <- -1
    return(l)
}

#' Read annotation data (*.chip)
#'
#' 'readChipFile' reads a tab-delimited text file containing annotation data 
#' about a microarray platform.
#' @param fileName a string containing the file name
#' @return a character matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, genes and samples.
#' @export
readChipFile <- function(fileName) {
    annotation <- read.csv(fileName, sep="\t", na.string=c("---", "", "NA"), 
                           header=TRUE)
    colnames(annotation) <- toupper(colnames(annotation))
    if (length(which(colnames(annotation) %in% c("PROBE.SET.ID", 
                                                 "GENE.SYMBOL"))) != 2)
        stop(paste("Wrong annotation data format. The file must contain",
                   "the columns \"Probe set ID\" and \"Gene Symbol\""))

    return(as.table(as.matrix(annotation)))
}

#' Collapse probe sets to gene symbols
#' 
#' 'collapseExprData' collapse probe sets to gene symbols
#' @param expr a numeric matrix containing gene expression data in which the 
#' the rows correspond to the probe sets and the columns correspond to the 
#' samples.
#' @param annotation a character matrix associating each probe set to a gene 
#' symbol. It must have the 'Probe.set.id' and 'Gene.symbol' columns.
#' @param method a function that collapses probe sets to gene symbols. 
#' It receives 'expr', the row groups, the row IDs, and 'options'
#' @param options a list containing parameters for the collapsing method. 
#' For the available methods, 'options' can be set to NULL, 
#' list("connectivityBasedCollapsing"=TRUE) or 
#' list("connectivityBasedCollapsing"=FALSE)
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
collapseExprData <- function(expr, annotation, method, options=NULL) {
    colnames(annotation) <- toupper(colnames(annotation))
    rownames(annotation) <- annotation[, "PROBE.SET.ID"]
    rowID <- rownames(expr)
    i <- which(rowID %in% annotation[, "PROBE.SET.ID"])
    if (length(i) < 1)
        stop("No gene from the expression data was found on the annotation file")
    rowID <- rowID[i]
    rowGroup <- annotation[rowID, "GENE.SYMBOL"]
    i <- which(!is.na(rowGroup))
    rowID <- rowID[i]
    rowGroup <- annotation[rowID, "GENE.SYMBOL"]
    return(method(expr, rowGroup, rowID, options))
}

#' Read a collection of gene sets (*.gmt)
#'
#' 'readGmtFile' reads a tab-delimited text file containing a collection of 
#' gene sets.
#' @param fileName a string containing the file name
#' @return a list of gene sets. Each element of the list is a character vector
#' v, where v[1] contains the gene set name, v[2] descriptions about the set,
#' v[3..length(v)] the genes that belong to the set.
#' @export
readGmtFile <- function(fileName) {
    lines <- readLines(fileName)
    geneSets <- strsplit(lines, "\t")
    n <- length(geneSets)
    for (i in 1:n) {
        if (length(geneSets[[i]]) < 3)
            stop(paste("Wrong gene sets file format. All gene sets should", 
                "contain at least one gene."))
    }
    return(geneSets)
}

#' CoGA differential network analysis
#'
#' 'diffNetAnalysis' performs CoGA differential expression analyses. 
#' @param method a method receives two adjacency matrices and returns a list
#' containing a statistic theta that measures the difference between them, and a 
#' a p-value for the test H0: theta = 0 against H1: theta > 0.
#' @param options a list contaning paremeters used by 'method'.
#' @param expr a numeric matrix contaning gene expression data.
#' @param labels a vector of -1s, 0s, and 1s associating each sample with a 
#' phenotype. The value 0 corresponds to the first phenotype class of interest,
#' 1 to the second phenotype class of interest, and -1 to the other classes, if
#' there are more than two classes in the gene expression data.
#' @param geneSets a list of gene sets. Each element of the list is a character 
#' vector v, where v[1] contains the gene set name, v[2] descriptions about the 
#' set, v[3..length(v)] the genes that belong to the set.
#' @param adjacencyMatrix a function that receives a numeric matrix 
#' containing gene expression data and returns the adjacency matrix of the 
#' inferred co-expression graph.
#' @param numPermutations the number of permutations for the permutation test. 
#' @param print a logical. If true, it prints execution messages on the screen.
#' resultsFile: path to a file where the partial results of the analysis will
#' be saved. If NULL, then no partial results are saved.
#' @param seed the seed for the random number generators. If it is not null
#' then the sample permutations are the same for all the gene sets. 
#' @return a data frame containing the name, size, test statistic, nominal
#' p-value and adjusted p-value (q-value) associated with each gene set. 
#' @export
diffNetAnalysis <- function(method, options, expr, labels, geneSets,
                            adjacencyMatrix, numPermutations=1000, print=TRUE, 
                            resultsFile=NULL, seed=NULL) {
    results <- data.frame(matrix(NA, nrow=length(geneSets), ncol=4))
    names <- array(NA, length(geneSets))
    for (i in 1:length(geneSets)) {
        names[i] <- geneSets[[i]][1]
    }
    rownames(results) <- names
    colnames(results) <- c("Set size", "Test statistic", "Nominal p-value", "Q-value")
    #temp <- tempfile("CoGA_results", fileext=".txt")
    for (i in 1:length(geneSets)) {
        setName <- geneSets[[i]][1]
        if (print)
            cat(paste("Testing ", setName, " (", i, " of ", length(geneSets), 
                      ")", "\n", sep=""), append=T)
        genes <- geneSets[[i]][geneSets[[i]] %in% rownames(expr)]
        if (!is.null(seed))
            set.seed(seed)
        result <- method(expr[genes, ], labels, adjacencyMatrix=
                          adjacencyMatrix,  numPermutations=
                          numPermutations, options=options)
        results[setName, "Test statistic"] <- result[[1]]
        results[setName, "Nominal p-value"] <- result$p.value
        results[setName, "Set size"] <- length(genes)
        if (!is.null(resultsFile))
            save(results, file=resultsFile)
    }
    results[, "Q-value"] <- p.adjust(results[, "Nominal p-value"], 
                                        method="fdr")
    #unlink(temp)
    return(results)
}

#' Run CoGA
#'
#' Run CoGA on the browser user interface.
#' @export
runCoga <- function() {
    shiny::runApp(system.file('shiny', package='coga'))
}
