# ------------------------------------------------------------------------------
# Collapsing methods
# ------------------------------------------------------------------------------

#' MaxMean method for collapsing probe sets to genes
#'
#' Choose the row with the highest mean value.
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
maxMean <- function(expr, rowGroup, rowID, options) {
    connectivityBasedCollapsing <- F
    if (!is.null(options)) {
        if (options$connectivityBasedCollapsing == "True")
            connectivityBasedCollapsing <- T 
    }
    collapseRows(expr, rowGroup, rowID, method="MaxMean", 
                 connectivityBasedCollapsing=
                 connectivityBasedCollapsing)$datETcollapsed
}

#' MinMean method for collapsing probe sets to genes
#'
#' Choose the row with the lowest mean value
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
minMean <- function(expr, rowGroup, rowID, options) {
    connectivityBasedCollapsing <- F
    if (!is.null(options)) {
        if (options$connectivityBasedCollapsing == "True")
            connectivityBasedCollapsing <- T 
    }
    collapseRows(expr, rowGroup, rowID, method="MinMean", 
                 connectivityBasedCollapsing=
                 connectivityBasedCollapsing)$datETcollapsed
}

#' AbsMaxMean method for collapsing probe sets to genes
#' 
#' Choose the row with the highest mean absolute value
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
absMaxMean <- function(expr, rowGroup, rowID, options) {
    connectivityBasedCollapsing <- F
    if (!is.null(options)) {
        if (options$connectivityBasedCollapsing == "True")
            connectivityBasedCollapsing <- T 
    }
    collapseRows(expr, rowGroup, rowID, method="absMaxMean", 
                 connectivityBasedCollapsing=
                 connectivityBasedCollapsing)$datETcollapsed
}

#' AbsMinMean method for collapsing probe sets to genes
#' 
#' Choose the row with the lowest mean absolute value
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
absMinMean <- function(expr, rowGroup, rowID, options) {
    connectivityBasedCollapsing <- F
    if (!is.null(options)) {
        if (options$connectivityBasedCollapsing == "True")
            connectivityBasedCollapsing <- T 
    }
    collapseRows(expr, rowGroup, rowID, method="absMinMean", 
                 connectivityBasedCollapsing=
                 connectivityBasedCollapsing)$datETcollapsed
}

#' MaxRowVariance method for collapsing probe sets to genes
#' Choose the row with the highest variance
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
maxRowVariance <- function(expr, rowGroup, rowID, options) {
    connectivityBasedCollapsing <- F
    if (!is.null(options)) {
        if (options$connectivityBasedCollapsing == "True")
            connectivityBasedCollapsing <- T 
    }
    collapseRows(expr, rowGroup, rowID, method="maxRowVariance", 
                 connectivityBasedCollapsing=
                 connectivityBasedCollapsing)$datETcollapsed
}

#' Me method for collapsing probe sets to genes
#' 
#' Choose the eigenrow (first principal component of the rows that represent the 
# same gene)
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
me <- function(expr, rowGroup, rowID, options) {
    collapseRows(expr, rowGroup, rowID, method="ME")$datETcollapsed
}

#' Average method for collapsing probe sets to genes
#' 
#' For each column, take the average value of the rows that represent the same 
# gene
#' @param expr: gene expression data
#' @param rowGroup:
#' @param rowID:
#' @param options: a list containing parameters for the collapsing method. 
#' It can be set to \code{NULL}, \code{list("connectivityBasedCollapsing"=TRUE)} 
#' or \code{list("connectivityBasedCollapsing"=FALSE)}. If the parameter is set
#' to true then the genes with 3 or more corresponding rows will be represented 
#' by the row with the highest connectivity according to a signed weighted
#' correlation network adjacency matrix among the corresponding rows.
#' @import WGCNA
#' @seealso \code{collapseRows}
#' @return a numeric matrix containing the gene expression collapsed data in 
#' which the rows and columns correspond to, respectively, the genes and 
#' the samples.
#' @export
average <- function(expr, rowGroup, rowID, options) {
    connectivityBasedCollapsing <- F
    if (!is.null(options)) {
        if (options$connectivityBasedCollapsing == "True")
            connectivityBasedCollapsing <- T 
    }
    collapseRows(expr, rowGroup, rowID, method="Average", 
                 connectivityBasedCollapsing=
                 connectivityBasedCollapsing)$datETcollapsed
}
