# ------------------------------------------------------------------------------
# Graph features
# ------------------------------------------------------------------------------

# ------------------------------------------------------------------------------
# Helper functions
# ------------------------------------------------------------------------------

clusterCoef <- function(A) {
    s <- sum(A)
    results <- c()
    n <- nrow(A)
    d <- (n-1)*(n-2)
    for (i in 1:n) {
        results[i] <- (s - 2*sum(A[i,]))/d
    }
    return(results)
}

#' Density functions of the degrees of two graphs
#'
#' 'degreeDensities' estimates the density functions of the degrees for two 
#' graphs at the same coordinates
#' @param G1 an igraph graph object
#' @param G2 an igraph graph object
#' @return a list containing the components, f1 (density estimate of the 
#' graph G1), and f2 (density estimate of the graph G2). Each component is
#' a list, where the first element is the vector 'x' of 'npoints' coordinates
#' of the points where the density function is estimated, and the second is 
#' a vector 'y' of the estimated density values.
#' @seealso \code{graph.strength}
#' @seealso \code{density}
#' @import igraph
#' @export
degreeDensities <- function(G1, G2, npoints=1024, options=list(bandwidth="Sturges")) {
    n1 <- vcount(G1)
    n2 <- vcount(G2)
    e1 <- graph.strength(G1)
    e2 <- graph.strength(G2)
    #b1 <- kernelBandwidth(e1)
    #b2 <- kernelBandwidth(e2)
    #from <- min(min(e1) - 3*b1, min(e2) - 3*b2)
    #to <- max(max(e1) + 3*b1, max(e2) + 3*b2)
    from <- min(e1, e2)
    to <- max(e1, e2)
    f1 <- gaussianDensity(e1, from=from, to=to, bandwidth=options$bandwidth, npoints=npoints)
    f2 <- gaussianDensity(e2, from=from, to=to, bandwidth=options$bandwidth, npoints=npoints)
    if (sum(is.na(f1)) > 0 || sum(is.na(f2)) > 0)
        return(NA)
    return(list("f1"=f1, "f2"=f2))
}

#' Jensen-Shannon divergence between the density functions of the degrees of
#' two graphs
#'
#' 'JSdegree' computes the Jensen-Shannon divergence between the density 
#' functions of the degrees of two graphs
#'
#' @param G1 an igraph graph object
#' @param G2 an igraph graph object
#' @param options a list containing parameters. It can be set to either 
#' \code{list(bandwidth="Sturges")} or \code{list(bandwidth="Silverman")}.
#' @return a list containing the components, f1 (density estimate of the 
#' graph G1), and f2 (density estimate of the graph G2). Each component is
#' a list, where the first element is the vector 'x' of 'npoints' coordinates
#' of the points where the density function i estimated, and the second is
#' a vector 'y' of the estimated density values.
#' @seealso \code{graph.strength}
#' @seealso \code{density}
#' @import igraph
#' @export
JSdegree <- function(G1, G2, options=list(bandwidth="Sturges")) {
    f <- degreeDensities(G1, G2, options=options)
    if (sum(is.na(f)) > 0)
        return(NA)
    f1 <- f$f1
    f2 <- f$f2
    fm <- f1
    fm$y <- (f1$y + f2$y)/2
    return((KL(f1, fm) + KL(f2, fm))/2)
}

# Returns the spectral density for a given adjacency matrix A
spectralDensity <- function(A, bandwidth="Sturges", npoints=1024) {
    eigenvalues <- (as.numeric(eigen(A, only.values = TRUE)$values)/sqrt(nrow(A))) 
    return(gaussianDensity(eigenvalues, bandwidth=bandwidth, npoints=npoints))
}

# Returns the spectral densities for given adjacency matrices A1 and A2 at the
# same points
#' @export
spectralDensities <- function(A1, A2, bandwidth="Sturges", 
                              npoints=1024) {
    n1 <- nrow(A1)
    n2 <- nrow(A2)
    e1 <- (as.numeric(eigen(A1, only.values = TRUE)$values)/sqrt(n1))
    e2 <- (as.numeric(eigen(A2, only.values = TRUE)$values)/sqrt(n2))
    #b1 <- kernelBandwidth(e1)
    #b2 <- kernelBandwidth(e2)
    #from <- min(min(e1) - 3*b1, min(e2) - 3*b2)
    #to <- max(max(e1) + 3*b1, max(e2) + 3*b2)
    from <- min(e1, e2)
    to <- max(e1, e2)
    f1 <- gaussianDensity(e1, from=from, to=to, bandwidth=bandwidth, npoints=npoints)
    f2 <- gaussianDensity(e2, from=from, to=to, bandwidth=bandwidth, npoints=npoints)
    if (sum(is.na(f1)) > 0 || sum(is.na(f2)) > 0)
        return(NA)
    return(list("f1"=f1, "f2"=f2))
}

# Given two adjacency matrices, returns the Jensen-Shannon divergence between
# the corresponding graphs
#' @export
JSspectrum <- function(A1, A2, bandwidth="Sturges") {
    f <- spectralDensities(A1, A2, bandwidth=bandwidth)
    if (sum(is.na(f)) > 0)
        return(NA)
    f1 <- f$f1
    f2 <- f$f2
    fm <- f1
    fm$y <- (f1$y + f2$y)/2
    return((KL(f1, fm) + KL(f2, fm))/2)
}

# Returns the absolute difference of the adjacency matrix A1 and A2 spectral 
# entropies
absDiffSpectralEntropy <- function(A1, A2, bandwidth="Sturges") {
    fs <- spectralDensities(A1, A2, bandwidth=bandwidth)
    if (sum(is.na(fs)) > 0)
        return(NA)
    H1 <- entropy(fs$f1)
    H2 <- entropy(fs$f2)
    return(abs(H1 - H2)) 
}


# ------------------------------------------------------------------------------
# Gene scores
# ------------------------------------------------------------------------------

#' Degree centrality
#' @export
degreeCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- list(graph.strength(G1), graph.strength(G2))
    return(result)
}

#' Betweenness centrality
#' @export
betweennessCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (!is.null(weighted)) {
        i1 <- which(A1 > 0)
        i2 <- which(A2 > 0)
        A1[i1] <- 1/A1[i1]
        A2[i2] <- 1/A2[i2]
    }
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- list(betweenness(G1), betweenness(G2))
    return(result)
}

#' Closeness centrality
#' @export
closenessCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (!is.null(weighted)) {
        i1 <- which(A1 > 0)
        i2 <- which(A2 > 0)
        A1[i1] <- 1/A1[i1]
        A2[i2] <- 1/A2[i2]
    }
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- list(closeness(G1), closeness(G2))
    return(result)
}

#' Eigenvector centrality
#' @export
eigenvectorCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- list(evcent(G1)$vector, evcent(G2)$vector)
    return(result)
}

#' Clustering coefficient
#' @export
clusteringCoefficient <- function(expr, labels, adjacencyMatrix, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- F
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0) {
        weighted <- T
    }
    if (weighted)
        result <- list(clusterCoef(A1), clusterCoef(A2))
    else {
        G1 <- graph.adjacency(A1, mode="undirected", weighted=NULL)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=NULL)
        result <- list(transitivity(G1, type="local", isolates="zero"), 
                       transitivity(G2, type="local", isolates="zero"))
    }
    return(result)
}

# ------------------------------------------------------------------------------
# Network features
# ------------------------------------------------------------------------------

#' Average degree centrality
#' @export
averageDegreeCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    result <- degreeCentrality(expr, labels, adjacencyMatrix)
    return(list(mean(result[[1]]), mean(result[[2]])))
}

#' Average betweenness centrality
#' @export
averageBetweennessCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    result <- betweennessCentrality(expr, labels, adjacencyMatrix)
    return(list(mean(result[[1]]), mean(result[[2]])))
}

#' Average closeness centrality
#' @export
averageClosenessCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    result <- closenessCentrality(expr, labels, adjacencyMatrix)
    return(list(mean(result[[1]]), mean(result[[2]]))) 
}

#' Average eigenvector centrality
#' @export
averageEigenvectorCentrality <- function(expr, labels, adjacencyMatrix, options=NULL) {
    result <- eigenvectorCentrality(expr, labels, adjacencyMatrix)
    return(list(mean(result[[1]]), mean(result[[2]]))) 
}

#' Average clustering coefficient
#' @export
averageClusteringCoefficient <- function(expr, labels, adjacencyMatrix, options=NULL) {
    result <- clusteringCoefficient(expr, labels, adjacencyMatrix)
    return(list(mean(result[[1]]), mean(result[[2]])))
}

#' Average shortest path
#' @export
averageShortestPath <- function(expr, labels, adjacencyMatrix, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (!is.null(weighted)) {
        i1 <- which(A1 > 0)
        i2 <- which(A2 > 0)
        A1[i1] <- 1/A1[i1]
        A2[i2] <- 1/A2[i2]
    }
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- list(average.path.length(G1), average.path.length(G2))
    return(result)
}

# Returns the spectral entropy for a given adjacency matrix A
spectralEntropy <- function(A, bandwidth="Sturges") {
    f <- spectralDensity(A, bandwidth=bandwidth)
    if (sum(is.na(f)) > 0)
        return(NA)
    y <- f$y
    n <- length(y)
    i <- which(y != 0)
    y[i] <- y[i]*log(y[i])
    return(-trapezoidSum(f$x, y))
}

#' Spectral entropies
#' @export
spectralEntropies <- function(expr, labels, adjacencyMatrix, options=list(bandwidth="Sturges")) {
    A1 <- adjacencyMatrix(expr[, which(labels == 0)])
    A2 <- adjacencyMatrix(expr[, which(labels == 1)])
    fs <- spectralDensities(A1, A2, bandwidth=options$bandwidth)
    if (sum(is.na(fs)) > 0)
        return(NA)
    H1 <- entropy(fs$f1)
    H2 <- entropy(fs$f2)
    return(list(H1, H2)) 
}

# ------------------------------------------------------------------------------
# Test of equality between network properties
# ------------------------------------------------------------------------------

#' Degree centrality test
#' @export
degreeCentralityTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    n <- nrow(expr)
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- dist(rbind(graph.strength(G1), graph.strength(G2)))[1]/sqrt(n)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
        results[i] <-  dist(rbind(graph.strength(G1), 
                            graph.strength(G2)))[1]/sqrt(n)
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

#' Betweenness centrality test
#' @export
betweennessCentralityTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    n <- nrow(expr)
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (!is.null(weighted)) {
        i1 <- which(A1 > 0)
        i2 <- which(A2 > 0)
        A1[i1] <- 1/A1[i1]
        A2[i2] <- 1/A2[i2]
    }
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- dist(rbind(betweenness(G1), betweenness(G2)))[1]/sqrt(n)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        if (!is.null(weighted)) {
            i1 <- which(A1 > 0)
            i2 <- which(A2 > 0)
            A1[i1] <- 1/A1[i1]
            A2[i2] <- 1/A2[i2]
        }
        G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
        results[i] <-  dist(rbind(betweenness(G1), betweenness(G2)))[1]/sqrt(n)
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

#' Closeness centrality test
#' @export
closenessCentralityTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    n <- nrow(expr)
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (!is.null(weighted)) {
        i1 <- which(A1 > 0)
        i2 <- which(A2 > 0)
        A1[i1] <- 1/A1[i1]
        A2[i2] <- 1/A2[i2]
    }
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- dist(rbind(closeness(G1), closeness(G2)))[1]/sqrt(n)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        if (!is.null(weighted)) {
            i1 <- which(A1 > 0)
            i2 <- which(A2 > 0)
            A1[i1] <- 1/A1[i1]
            A2[i2] <- 1/A2[i2]
        }
        G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
        results[i] <- dist(rbind(closeness(G1), closeness(G2)))[1]/sqrt(n)
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

#' Eigenvector centrality test
#' @export
eigenvectorCentralityTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    n <- nrow(expr)
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- dist(rbind(evcent(G1)$vector, evcent(G2)$vector))[1]/sqrt(n)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
        results[i] <-  dist(rbind(evcent(G1)$vector, 
                            evcent(G2)$vector))[1]/sqrt(n)
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

#' Clustering coefficient test
#' @export
clusteringCoefficientTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    n <- nrow(expr)
    weighted <- F
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (weighted)
        result <- dist(rbind(clusterCoef(A1), clusterCoef(A2)))[1]/sqrt(n)
    else {
        G1 <- graph.adjacency(A1, mode="undirected", weighted=NULL)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=NULL)  
        result <- dist(rbind(transitivity(G1, type="local", isolates="zero"),
                            transitivity(G2, type="local", 
                                         isolates="zero")))[1]/sqrt(n)
    }
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        if (weighted)
            results[i] <-dist(rbind(clusterCoef(A1), 
                                    clusterCoef(A2)))[1]/sqrt(n)
        else {
            G1 <- graph.adjacency(A1, mode="undirected", weighted=NULL)
            G2 <- graph.adjacency(A2, mode="undirected", weighted=NULL)
            results[i] <- dist(rbind(transitivity(G1, type="local", 
                                                  isolates="zero"),
                                     transitivity(G2, type="local", 
                                                  isolates="zero")))[1]/sqrt(n)
        }
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

#' Shortest path test
#' @export
shortestPathTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=NULL) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    if (!is.null(weighted)) {
        i1 <- which(A1 > 0)
        i2 <- which(A2 > 0)
        A1[i1] <- 1/A1[i1]
        A2[i2] <- 1/A2[i2]
    }
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
    result <- abs(average.path.length(G1)-average.path.length(G2))
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        if (is.null(weighted)) {
            i1 <- which(A1 > 0)
            i2 <- which(A2 > 0)
            A1[i1] <- 1/A1[i1]
            A2[i2] <- 1/A2[i2]
        }
        G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
        results[i] <-  abs(average.path.length(G1)-average.path.length(G2))
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

#' Degree distribution test
#' @export
degreeDistributionTest <- function(expr, labels, adjacencyMatrix, 
                           numPermutations=1000, options=list(bandwidth="Sturges")) {
    A1 <- adjacencyMatrix(expr[, labels==0])
    A2 <- adjacencyMatrix(expr[, labels==1])
    weighted <- NULL
    if (sum(!(A1 %in% c(1,0))) != 0 || sum(!(A2 %in% c(1,0))) != 0)
        weighted <- T
    G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
    G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)     
    result <- JSdegree(G1, G2, options=options)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, l==0])
        A2 <- adjacencyMatrix(expr[, l==1])
        G1 <- graph.adjacency(A1, mode="undirected", weighted=weighted)
        G2 <- graph.adjacency(A2, mode="undirected", weighted=weighted)
        results[i] <- JSdegree(G1, G2, options=options)
    }
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("measure"=result, "p.value"=pvalue))
}

# ------------------------------------------------------------------------------

# Test for the absolute difference between the entropies of two graphs (genes  
# networks), described by:
# H0: |H(A1) - H(A2)| = 0
# H1: |H(A1) - H(A2)| > 0
# where A1 and A2 are the adjacency matrices of the graphs
#
# -- Input: --
# "expr" - Matrix of genes (rows) vs microarrays (columns)
# "labels" - binary vector in which a position indicates the phenotype (0 or 1) 
# of the corresponding microarray
# "adjacencyMatrix" - a function that returns the adjacency matrix for a given 
# gene expression matrix
# "numPermutations" - number of permutations that will be carried out in the
# permutation test
# 
# -- Output: --
# A list containing: 
# "abs.diff" - the absolute difference between the entropies of the two gene 
# networks associated with each phenotype (0 or 1)
# "p.value" - the Nominal p-value of the test for the Jensen-Shannon divergence

#' Spectral entropy test
#' @export
spectralEntropyTest <- function(expr, labels, adjacencyMatrix, numPermutations=1000,
                                options=list(bandwidth="Sturges")) {
    A1 <- adjacencyMatrix(expr[, which(labels == 0)])
    A2 <- adjacencyMatrix(expr[, which(labels == 1)])
    #result <- spectralEntropyAbsDiff(A1, A2)
    result <- absDiffSpectralEntropy(A1, A2, bandwidth=options$bandwidth)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, which(l == 0)])
        A2 <- adjacencyMatrix(expr[, which(l == 1)])
        #results[i] <- spectralEntropyAbsDiff(A1, A2)
        results[i] <- absDiffSpectralEntropy(A1, A2, bandwidth=options$bandwidth)
    }
    #pvalue <- 1 - rank(c(result, results, recursive=TRUE), ties="first")[1]/
    #              (numPermutations + 1)
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("abs.diff"=result, "p.value"=pvalue))
}

# Test for the Jensen-Shannon divergence between two graphs (genes networks), 
# described by:
# H0: JS(A1, A2) = 0
# H1: JS(A1, A2) > 0
# where A1 and A2 are the adjacency matrices of the graphs
#
# -- Input: --
# "expr" - Matrix of genes (rows) vs microarrays (columns)
# "labels" - binary vector in which a position indicates the phenotype (0 or 1) 
# of the corresponding microarray
# "adjacencyMatrix" - a function that returns the adjacency matrix for a given 
# gene expression matrix (the "expr" matrix)
# "numPermutations" - number of permutations that will be carried out in the
# permutation test
# 
# -- Output: --
# A list containing: 
# "JS" - The Jensen-Shannon divergence between the gene networks associated with 
# each phenotype (0 or 1)
# "p.value" - the Nominal p-value of the test for the Jensen-Shannon divergence

#' Spectral distribution test
#' @export
spectralDistributionTest <- function(expr, labels, adjacencyMatrix, 
                                     numPermutations=1000,
                                     options=list(bandwidth="Sturges")) {
    A1 <- adjacencyMatrix(expr[, which(labels == 0)])
    A2 <- adjacencyMatrix(expr[, which(labels == 1)])
    result <- JSspectrum(A1, A2, bandwidth=options$bandwidth)
    results <- vector(len=numPermutations)
    for (i in 1:numPermutations) {
        l <- sample(labels, replace = FALSE)
        A1 <- adjacencyMatrix(expr[, which(l == 0)])
        A2 <- adjacencyMatrix(expr[, which(l == 1)])
        results[i] <- JSspectrum(A1, A2, bandwidth=options$bandwidth)
    }
    #pvalue <- 1 - rank(c(result, results, recursive=TRUE), ties="first")[1]/
    #              (numPermutations + 1)
    pvalue <- (1 + sum(results >= result))/(numPermutations + 1)
    return(list("JS"=result, "p.value"=pvalue))
}
