#' 'adjacencyMatrix' creates a function that infers a graph from gene expression 
#' data.
#' @param method: a function that measures the association between the gene 
#' expression levels.
#' @param abs: a logical value. If TRUE, then it uses the absolute value 
#' returned by 'method' as a measure of association between the gene products.
#' @param pvalue: a logical value. If TRUE, then it uses one minus the value
#' returned by 'method' as a measure of association between the gene products.
#' @param fdr: a logical value. If TRUE, then it assumes that the value returned
#' by 'method' is a p-value, and adjust it by the False Discovery Rate 
#' (Benjamini and Hochberg, 1995) method for multiple testing. This parameter
#' is ignored if 'pvalue' is FALSE.
#' @param weighted: a logical value. If TRUE, then the edges of the graph are 
#' weighted by the association degrees between the gene products. Otherwise,
#' the edges are removed according to a threshold, and the remaining links are
#' weighted by one.
#' @param threshold: a numeric value. If NULL, no edge is removed. Otherwise
#' the function removes all edges weighted by a value less than or equal to 
#' 'threshold'. It is ignored of the 'weighted' is TRUE.
#' @return a function that creates an adjacency matrix from gene expression 
#' data.
#' @export
adjacencyMatrix <- function(method, abs=T, pvalue=T, fdr=T, weighted=T, threshold=NULL) {
    return(
        function(expr) {
            A <- method(expr)
            if (abs)
                A <- abs(A)
            diag(A) <- 0
            if (pvalue) {
                if (fdr) {
                    A[lower.tri(A)] <- p.adjust(A[lower.tri(A)], method="fdr")
                    A[upper.tri(A)] <- 0
                    A <- A + t(A)
                }
                diag(A) <- 1
                A <- 1 - A
            }
            if (!weighted) {
                edges <- which(A > threshold)
                remove <- which(A <= threshold)
                A[edges] <- 1
                A[remove] <- 0
            }
            return(A)
        }
    )
}

#' @export
pearsonCor <- function(expr) {
    expr <- t(expr)
    A <- cor(expr, method="pearson")
    return(A)
}

#' @export
pearsonTest <- function(expr) {
    expr <- t(expr)
    A <- Hmisc::rcorr(expr, type="pearson")$P
    return(A)
}

#' @export
spearmanCor <- function(expr) {
    expr <- t(expr)
    A <- cor(expr, method="spearman")
    return(A)
}

#' @export
spearmanTest <- function(expr) {
    expr <- t(expr)
    A <- Hmisc::rcorr(expr, type="spearman")$P
    return(A)
}

#' @export
kendallCor <- function(expr) {
    expr <- t(expr)
    A <- cor(expr, method="kendall")
    return(A)
}

#' @export
kendallTest <- function(expr) {
    expr <- t(expr)
    A <- psych::corr.test(expr, method="kendall", adjust="none")$p
    return(A)
}      
        