# ------------------------------------------------------------------------------
# CoGA User Interface (UI) definition
# ------------------------------------------------------------------------------

# Define UI for application 
shinyUI(pageWithSidebar(

    # Application title --------------------------------------------------------
    headerPanel(
        p(img(src="images/CoGA.png"), align="center"),
        windowTitle="CoGA" 
    ),

    # Sidebar to load data and set application parameters ----------------------
    sidebarPanel(
        # Input files
        h3("1. Load data"),
        wellPanel(
            h5("Gene expression data"),
            fileInput("expr", 
                      paste("Please, select the text file (*.txt) containing",
                            "the expression data"),
                      accept=c("text", "text/plain", ".txt"))
        ),
        wellPanel(
            h5("Annotation data"),
            radioButtons(
                "annotationOptions", 
                "",
                c("Keep the original expression data rows"="nocollapse",
                  "Collapse the dataset to gene symbols"="collapse")
            ),
            conditionalPanel(
                "input.annotationOptions=='collapse'",
                uiOutput("collapsingMethod"),
                uiOutput("collapsingMethodDescription"),
                uiOutput("collapsingMethodOptions"),
                fileInput("annotation", 
                          paste("Load the chip file (*.chip) which maps the", 
                                "microarray probe sets (dataset rows) to gene",
                                "symbols"),
                          accept=c(".CHIP", ".chip"))
            )
        ),
        wellPanel( 
            h5("Sample labels"),
            fileInput("labels", 
                      paste("Load the categorical class file (*.cls)", 
                            "describing which class (condition) each sample",
                            "belongs to"),
                      accept=c('.cls'))
        ),
        wellPanel(
            h5("Gene set database"),
            fileInput("geneSets", 
                      paste("Load the gene matrix transposed file (*.gmt)", 
                            "describing the gene sets"),
                      accept=c('.gmt'))
        ),

        # Parameters
        h3("2. Set parameters"),
        wellPanel(
            h5("Classes (conditions) being compared"),
            uiOutput("classes")
        ),
        wellPanel(
            h5("Gene sets size range"),
            p("Enter the minimum and maximum gene set sizes."),
            uiOutput("minSize"),
            uiOutput("maxSize"),
            uiOutput("geneSetsCount")
        ),
        wellPanel(
            h5("Method for network inference"),
            uiOutput("correlationMeasure"),
            radioButtons(
                "associationMeasure", 
                "",
                c("Absolute correlation"="correlation", "1 - p-value"="pvalue",
                  "1 - q-value"="qvalue")
            )
        ),
        wellPanel(
            h5("Network type"),
            radioButtons(
                "networkType",
                "",
                c("Weighted"="weighted", "Unweighted"="unweighted")
            ),
            conditionalPanel(
                "input.networkType=='unweighted'",
                numericInput("edgeThreshold", 
                             paste("Enter a association degree threshold to", 
                                   "define the network edges:"), 
                             0.95, min=0, max=1, step=0.01)
            )
        ),
        wellPanel(
            h5("Method for gene networks comparison"),
              uiOutput("networkTest"),
              uiOutput("networkTestDescription"),
              uiOutput("networkTestOptions")
        ), 
        wellPanel(
            h5("Permutation test settings"),
            numericInput("numPermutations", 
                         "Enter the number of label permutations:", 
                          1000, min=0),
            numericInput("seed", 
                         "Enter a seed to generate random permutations:", 0)
        ),
        h3("3. Run differential network analysis"),
        wellPanel(
           actionButton("start", "Start analysis")
        )
    ),


    # Main panel ---------------------------------------------------------------
    mainPanel(
        h3("Reports"),

        # Tabset
        tabsetPanel(
            id="tabSelected",
            
            # Data tab
            tabPanel(
                 h5("Loaded data"),
                 h5("Gene expression data"),
                 uiOutput("expr"), 
                 h5("Sample labels"),
                 uiOutput("labels"),
                 h5("Gene sets collection"), 
                 uiOutput("geneSets"),
                 value = "Loaded data"
            ),

            # Results tab
            tabPanel(
                h5("Analysis results"),
                bsAlert(inputId = "resultsWarning"),
                progressInit(),
                uiOutput("appMessages"),
                uiOutput("isCompleted"),
                wellPanel(
                    uiOutput("resultsType"),
                    uiOutput("downloadResultsButton")
                ),
                chartOutput('results', 'datatables'),
                value = "Analysis results"
            ),

            # Further analysis tab
            tabPanel(
                h5("Further analyses"),
                wellPanel(
                    h5("Gene set selection"),
                    div(
                        class="row-fluid",
                        div(class="span6", uiOutput("filterGeneSets"),
                            uiOutput("geneSetThreshold")), 
                        div(class="span6", uiOutput("selectGeneSet"))
                    ),
                    uiOutput("geneSetInfo")
                ),

                br(),
                uiOutput("selectedGeneSet"),
                br(),

                tabsetPanel(
                    id="FurtherTabSelected",
                    tabPanel(
                        "Network visualization plots",
                        bsCollapsePanel(
                            "Plot settings", 
                            div(
                                class="row-fluid",
                                div(
                                    class="span4",
                                    h5("Colors selection"),
                                    uiOutput("heatmapColors")
                                ),
                                div(
                                    class="span4",
                                    h5("Plot format"),
                                    radioButtons(
                                        "networkPlotFormat", 
                                        "Select a format to save the plots:",
                                        c("PDF","PNG", "JPG"))
                                ),
                                div(
                                    class="span4",
                                    h5("Plot dimensions"),
                                    uiOutput("networkPlotDimensions")
                                )
                            )
                        ),
                        bsCollapsePanel(
                            "Network visualization",
                            conditionalPanel(
                                "input.associationMeasure=='correlation'",
                                checkboxInput("signedCorrelation", 
                                              "Show negative correlations")
                            ),
                            br(),
                            div(class="row-fluid", 
                                div(class="span6", plotOutput("heatmapClass1"),
                                    uiOutput("downloadNetworkPlot1Button")), 
                                div(class="span6", plotOutput("heatmapClass2"),
                                    uiOutput("downloadNetworkPlot2Button"))),
                            br(),
                            wellPanel(
                                h5("Association between two gene products"),
                                uiOutput("selectGenes"),
                                tableOutput("corr")
                            )
                        ),
                        bsCollapsePanel(
                            "Differences between the gene networks",
                            uiOutput("heatmapDiffOptions"),
                            br(),
                            bsCollapsePanel(
                                "Matrix of differences",
                                plotOutput("heatmapDiff", width="50%"),
                                uiOutput("downloadNetworkDiffPlotButton")),
                            bsCollapsePanel(
                               "List of gene association degrees",
                                wellPanel(
                                    uiOutput("absDiffType"),
                                    uiOutput("downloadAbsDiffButton")
                                ),
                                br(),
                                chartOutput("corAbsDiff", "datatables")
                            )
                        )
                    ),
                    tabPanel(
                        "Gene set properties",
                        wellPanel(
                            h5("Gene set network topological properties"),
                            uiOutput("networkScore"),
                            uiOutput("networkScoreOptions"),
                            uiOutput("networkScoresComparison")
                        )
                    ),
                    tabPanel(
                        "Gene scores",
                        wellPanel(
                            h5("Gene scores"),
                            uiOutput("geneScore"),
                            uiOutput("geneScoresType"),
                            uiOutput("downloadGeneScoresButton")
                        ),
                        br(),
                        chartOutput("geneScoresComparison", "datatables")
                    ),
                    tabPanel(
                        "Gene expression analysis",
                        bsCollapsePanel(
                            "Gene expression heatmap",
                            wellPanel(
                                div(
                                    class="row-fluid",
                                    div(class="span4",
                                        h5("Colors selection"),
                                        uiOutput("exprHeatmapColors"),
                                        h5("Heatmap clustering options"),
                                        uiOutput("exprHeatmapClustering")),
                                    div(
                                        class="span4",
                                        h5("Plot format"),
                                        radioButtons(
                                            "exprHeatmapFormat", 
                                            "Select a format to save the plots:",
                                            c("PDF","PNG", "JPG"))
                                    ),
                                    div(class="span4",
                                        h5("Plot dimensions"),
                                        uiOutput("exprHeatmapDimensions"),
                                        uiOutput(paste("downloadExpr", 
                                                       "HeatmapButton", 
                                                        sep="")))
                                )
                            ),
                            plotOutput("exprHeatmap", width="100%")
                        ),
                        bsCollapsePanel(
                            "Tests for differential expression",  
                            wellPanel(
                                uiOutput("diffExprTestsType"),
                                uiOutput("downloadDiffExprTestsButton")
                            ),
                            br(),
                            chartOutput("diffExprTests", "datatables")
                        ),
                        bsCollapsePanel(
                            "Gene expression boxplot",  
                            wellPanel(
                                h5("Gene selection"),
                                uiOutput("selectGene")
                            ),
                            wellPanel(
                                div(class="row-fluid",
                                    div(class="span6",
                                        h5("Plot format"),
                                        radioButtons(
                                            "exprBoxplotFormat", 
                                            "Select a format to save the plots:",
                                            c("PDF","PNG", "JPG"))),
                                    div(class="span6",
                                        h5("Plot dimensions"),
                                        uiOutput("exprBoxplotDimensions"),
                                        uiOutput("downloadExprBoxplotButton")))
                            ),
                            br(),
                            plotOutput("exprBoxplot")
                        )
                    )
                ),
                value = "Further analyses"
            ),       

            # Help tab
            tabPanel(
                h5("Help"), 
                bsCollapse(
                    multiple=TRUE, 
                    id="help",
                    bsCollapsePanel(
                        "1. Loading the dataset", 
                        includeHTML("help/helpData.html"), 
                        id="helpData"
                    ),
                    bsCollapsePanel(
                        "2. Setting parameters and running the analysis", 
                        includeHTML("help/helpParameters.html"), 
                        id="helpParameters"
                    ),
                    bsCollapsePanel(
                        "3. Interpreting the results", 
                        includeHTML("help/helpResults.html"), 
                        id="helpResults"
                    ),
                    bsCollapsePanel(
                        "4. Further Analyses", 
                        includeHTML("help/helpFurtherAnalyses.html"), 
                        id="helpFurtherAnalyses"
                    ),
                    bsCollapsePanel(
                        "5. Running CoGA from the command-line interface", 
                        includeHTML("help/helpCommandLine.html"), 
                        id="helpCommandLine"
                    ),
                    bsCollapsePanel(
                        "6. References", 
                        includeHTML("help/helpReferences.html"), 
                        id="helpReferences"
                    )
                ),
                value = "Help"            
            ),
            # About tab
            tabPanel(h5("About"), includeHTML("about.html"), value="About")
        )
    )
))

